from ._C_netket.variational import *

import itertools


def _Vmc_iter(self, n_iter=None, step_size=1):
    """
    iter(self: Vmc, n_iter: int=None, step_size: int=1) -> int

    Returns a generator which advances the VMC optimization, yielding
    after every step_size steps up to n_iter.

    Args:
        n_iter (int=None): The number of steps or None, for no limit.
        step_size (int=1): The number of steps the simulation is advanced.

    Yields:
        int: The current step.
    """
    self.reset()
    for i in itertools.count(step=step_size):
        if n_iter and i >= n_iter:
            return
        self.advance(step_size)
        yield i


Vmc.iter = _Vmc_iter



from ._C_netket.operator import *
import numpy as _np


def Ising(hilbert, h, J=1.0):
    """
    Constructs a new ``Ising`` given a hilbert space, a transverse field,
    and (if specified) a coupling constant.

    Args:
        hilbert: Hilbert space the operator acts on.
        h: The strength of the transverse field.
        J: The strength of the coupling. Default is 1.0.

    Examples:
        Constructs an ``Ising`` operator for a 1D system.

        ```python
        >>> import netket as nk
        >>> g = nk.graph.Hypercube(length=20, n_dim=1, pbc=True)
        >>> hi = nk.hilbert.Spin(s=0.5, graph=g)
        >>> op = nk.operator.Ising(h=1.321, hilbert=hi, J=0.5)
        >>> print(op.hilbert.size)
        20
    """
    sigma_x = _np.array([[0, 1], [1, 0]])
    sz_sz = _np.array([[1, 0, 0, 0], [0, -1, 0, 0], [0, 0, -1, 0], [0, 0, 0, 1]])
    return GraphOperator(hilbert, siteops=[-h * sigma_x], bondops=[J * sz_sz])


def Heisenberg(hilbert):
    """
    Constructs a new ``Heisenberg`` given a hilbert space.
    Args:
        hilbert: Hilbert space the operator acts on.
    Examples:
     Constructs a ``Heisenberg`` operator for a 1D system.
        ```python
        >>> import netket as nk
        >>> g = nk.graph.Hypercube(length=20, n_dim=1, pbc=True)
        >>> hi = nk.hilbert.Spin(s=0.5, total_sz=0, graph=g)
        >>> op = nk.operator.Heisenberg(hilbert=hi)
        >>> print(op.hilbert.size)
        20
    """
    sz_sz = _np.array([[1, 0, 0, 0], [0, -1, 0, 0], [0, 0, -1, 0], [0, 0, 0, 1]])
    exchange = _np.array([[0, 0, 0, 0], [0, 0, 2, 0], [0, 2, 0, 0], [0, 0, 0, 0]])
    if hilbert.graph.is_bipartite:
        heis_term = sz_sz - exchange
    else:
        heis_term = sz_sz + exchange
    return GraphOperator(hilbert, bondops=[heis_term])



from ._C_netket.machine import *


def MPSPeriodicDiagonal(hilbert, bond_dim, symperiod=-1):
    return MPSPeriodic(hilbert, bond_dim, diag=True, symperiod=symperiod)


class CxxMachine(Machine):
    def __init__(self, hilbert):
        super(CxxMachine, self).__init__(hilbert)

    def log_val(self, v):
        r"""Returns the logarithm of the wave function for a visible
        configuration `v`.

        Subclasses should implement this function.

        Args:
            x: 1D vector of `float` of size `self.n_visible`.
        """
        raise NotImplementedError

    def der_log(self, v):
        r"""Returns the gradient of the logarithm of the wave function for
        a visible configuration `v`.

        Subclasses should implement this function.

        Args:
            x: 1D vector of `float` of size `self.n_visible`.
        """
        raise NotImplementedError

    def save(self, filename):
        r"""Saves machine's state to file.

        Subclasses should implement this function.

        *Note:* the sequence of saving and loading, -- ``m.save(filename);
        m.load(filename)``, -- should be an identity.

        Args:
            filename: name of the file.
	"""
        raise NotImplementedError

    def load(self, filename):
        r"""Loads machine's state from file.

        *Note:* the sequence of saving and loading, -- ``m.save(filename);
        m.load(filename)``, -- should be an identity.

        Subclasses should implement this function.

        Args:
            filename: name of the file.
	"""
        raise NotImplementedError

    def _is_holomorphic(self):
        r"""Returns whether the wave function is holomorphic.
        """
        raise NotImplementedError

    def _number_parameters(self):
        r"""Returns the number of variational parameters in the machine.

        Subclasses should implement this function, but to actually get the
        number of parameters, use `self.n_par` attribute.
        """
        raise NotImplementedError

    def _number_visible(self):
        r"""Returns the number of "visible units" in the machine.

        Subclasses should implement this function, but to actually get the
        dimension of input vector, use `self.n_visible` attribute.
        """
        raise NotImplementedError

    def _get_parameters(self):
        r"""Returns the variational parameters as a 1D complex vector.

        Subclasses should implement this function, but to actually access
        the parameters, use `self.parameters` attribute.
        """
        raise NotImplementedError

    def _set_parameters(self, p):
        r"""Sets the variational parameters.

        Subclasses should implement this function, but to actually access
        the parameters, use `self.parameters` attribute.

        Args:
            p: New variational parameters as a 1D complex vector.
        """
        raise NotImplementedError
